# Monocyte - Search and Destroy unwanted AWS Resources relentlessly.
# Copyright 2015 Immobilien Scout GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
from boto import ec2
from boto.exception import EC2ResponseError
from monocyte.handler import Resource, Handler


class Instance(Handler):
    VALID_TARGET_STATES = ["terminated", "shutting-down"]

    def fetch_regions(self):
        return ec2.regions()

    def fetch_unwanted_resources(self):
        for region in self.regions:
            connection = ec2.connect_to_region(region.name)
            resources = connection.get_only_instances() or []
            for resource in resources:
                resource_wrapper = Resource(resource=resource,
                                            resource_type=self.resource_type,
                                            resource_id=resource.id,
                                            creation_date=resource.launch_time,
                                            region=region.name)
                if resource.id in self.ignored_resources:
                    self.logger.info('IGNORE ' + self.to_string(resource_wrapper))
                    continue
                yield resource_wrapper

    def to_string(self, resource):
        return "ec2 instance found in {region.name}, " \
               "with identifier {id}, instance type is {instance_type}, created {launch_time}, " \
               "dnsname is {public_dns_name}, key {key_name}, with state {_state}".format(**vars(resource.wrapped))

    def delete(self, resource):
        if resource.wrapped.state in Instance.VALID_TARGET_STATES:
            raise Warning("state '{0}' is a valid target state, skipping".format(
                resource.wrapped.state))
        connection = ec2.connect_to_region(resource.region)
        if self.dry_run:
            try:
                connection.terminate_instances([resource.wrapped.id], dry_run=True)
            except EC2ResponseError as exc:
                if exc.status == 412:  # Precondition Failed
                    raise Warning("Termination {message}".format(**vars(exc)))
                raise
        else:
            instances = connection.terminate_instances([resource.wrapped.id], dry_run=False)
            self.logger.info("Initiating shutdown sequence for {0}".format(instances))
            return instances


class Volume(Handler):

    def fetch_regions(self):
        return ec2.regions()

    def fetch_unwanted_resources(self):
        for region in self.regions:
            connection = ec2.connect_to_region(region.name)
            resources = connection.get_all_volumes() or []
            for resource in resources:
                resource_wrapper = Resource(resource=resource,
                                            resource_type=self.resource_type,
                                            resource_id=resource.id,
                                            creation_date=resource.create_time,
                                            region=region.name)
                if resource.id in self.ignored_resources:
                    self.logger.info('IGNORE ' + self.to_string(resource_wrapper))
                    continue
                yield resource_wrapper

    def to_string(self, resource):
        return "ebs volume found in {region.name}, " \
               "with identifier {id}, created {create_time}, " \
               "with state {status}".format(**vars(resource.wrapped))

    def delete(self, resource):
        connection = ec2.connect_to_region(resource.region)

        if self.dry_run:
            try:
                connection.delete_volume(resource.wrapped.id, dry_run=True)
            except EC2ResponseError as exc:
                if exc.status == 412:  # Precondition Failed
                    warnings.warn(Warning("Termination {message}".format(**vars(exc))))
                raise
        else:
            self.logger.info("Initiating deletion of EBS volume {0}".format(resource.wrapped.id))
            connection.delete_volume(resource.wrapped.id, dry_run=False)



from __future__ import print_function, absolute_import, division
import yamlreader
import logging

from monocyte import Monocyte


def read_config(path):
    return {} if path is None else yamlreader.yaml_load(path)


def convert_arguments_to_config(arguments):
    dry_run = (arguments["--dry-run"] != "False")
    config_path = arguments["--config-path"]

    config = {
        "dry_run": dry_run,
    }

    return config_path, config


def apply_default_config(config):
    if config.get("cloudwatchlogs"):
        cloudwatchlogs_default_config = {
            'region': 'eu-central-1',
            'log_level': 'INFO',
            'groupname': 'monocyte_logs'
        }
        yamlreader.data_merge(cloudwatchlogs_default_config, config['cloudwatchlogs'])

        log_level_map = {
            'DEBUG': logging.DEBUG,
            'INFO': logging.INFO,
            'WARN': logging.WARN,
            'ERROR': logging.ERROR
        }
        cloudwatchlogs_default_config['log_level'] = log_level_map[cloudwatchlogs_default_config['log_level'].upper()]

        config['cloudwatchlogs'] = cloudwatchlogs_default_config

    default_config = {
        "handler_names": [
            "cloudformation.Stack",
            "ec2.Instance",
            "ec2.Volume",
            "rds2.Instance",
            "rds2.Snapshot",
            "dynamodb.Table",
            "s3.Bucket"],
        "ignored_resources": {"cloudformation": ["cloudtrail-logging"]},
        "ignored_regions": ["cn-north-1", "us-gov-west-1"],
        "allowed_regions_prefixes": ["eu"]
    }
    for key in default_config.keys():
        config[key] = config.get(key, default_config[key])


def main(arguments):
    path, cli_config = convert_arguments_to_config(arguments)
    file_config = read_config(path)
    config = yamlreader.data_merge(file_config, cli_config)
    apply_default_config(config)

    monocyte = Monocyte(**config)

    return monocyte.search_and_destroy_unwanted_resources()



#!/usr/bin/env python

from setuptools import setup

if __name__ == '__main__':
    setup(
          name = 'aws-monocyte',
          version = '0.3.381-139',
          description = '''Monocyte - Search and Destroy unwanted AWS Resources relentlessly.''',
          long_description = '''
    Monocyte is a bot for destroying AWS resources in non-EU regions written in Python using Boto.
    It is especially useful for companies that are bound to European privacy laws
    and for that reason don't want to process user data in non-EU regions.
    ''',
          author = "Jan Brennenstuhl, Arne Hilmann",
          author_email = "jan@brennenstuhl.me, arne.hilmann@gmail.com",
          license = 'Apache License 2.0',
          url = 'https://github.com/ImmobilienScout24/aws-monocyte',
          scripts = ['scripts/monocyte'],
          packages = ['monocyte', 'monocyte.plugins', 'monocyte.handler'],
          py_modules = [],
          classifiers = ['Development Status :: 4 - Beta', 'Environment :: Console', 'Intended Audience :: Developers', 'Intended Audience :: System Administrators', 'Programming Language :: Python', 'Topic :: System :: Networking', 'Topic :: System :: Software Distribution', 'Topic :: System :: Systems Administration'],
          entry_points={
          'console_scripts':
              []
          },
             #  data files
             # package data
          install_requires = [ "boto", "docopt", "pils", "python-cloudwatchlogs-logging", "yamlreader" ],
          
          zip_safe=True
    )
